<?php

namespace SM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use SM\Service\SmPropostasDespesasService;
use SM\Traits\TResponse;

class SmPropostasDespesasController
{
    use TResponse;
    protected $propostaDespesaservice;

    public function __construct(SmPropostasDespesasService $propostaDespesaservice)
    {
        $this->propostaDespesaservice = $propostaDespesaservice;
    }

    public function getDespesasproposta()
    {
        try {
            $data = $this->propostaDespesaservice->getDespesasproposta();
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    } 

    public function saveDespesas(DTO\Request $request)
    {
        try {
            $data = $this->propostaDespesaservice->saveDespesas($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteFormacaoPreco(DTO\Request $request)
    {
        try {
            $data = $this->propostaDespesaservice->deleteFormacaoPreco($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteRelacCompDespesas(DTO\Request $request)
    {
        try {
            $data = $this->propostaDespesaservice->deleteRelacCompDespesas($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function deleteDespesas(DTO\Request $request)
    {
        try {
            $data = $this->propostaDespesaservice->deleteDespesas($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getComposicaoDespesas(Request $request) 
    {
        try {
            $parameters = $request->getParameters();
            $data = $this->propostaDespesaservice->getComposicaoDespesas($parameters);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    } 

    public function getTabFormacaoPreco(Request $request) 
    {
        try {
            $parameters = $request->getParameters();
            $data = $this->propostaDespesaservice->getTabFormacaoPreco($parameters);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }  

    public function saveComposicaoServicosDespesas(Request $request) 
    {
        try {
            $params = $request->getParameters();
            $data = $this->propostaDespesaservice->saveComposicaoServicos($params);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }    

    public function saveRelacFormDesp(Request $request) 
    {
        try {
            $parameters = $request->getParameters();
            $data = $this->propostaDespesaservice->saveRelacFormDesp(  $parameters);
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }  


    public function deleteImposto(Request $request)
    {
        try {
            $data = $this->propostaImpostoService->deleteImposto($request->getParameters());
            $this->sendResponse($data, 200);
        } catch (\Exception $ex) {
            $this->sendResponse(["error" => $ex->getMessage(), "code" => $ex->getCode()], 500);
        }
    }
    
    /**
     * Deleta alguns servicos do imposto ou o imposto inteiro
    **/
    // public function deleteImposto(Request\Row $request, Response $response){
    //     try{
    //         $row = $request->getRow();
    //         $cdimposto     = $row["CDIMPOSTO"];
    //         $nrimpostoserv = $row["NRIMPOSTOSERV"];
    //         $this->propostaImpostoService->deleteImposto($cdimposto, $nrimpostoserv);
    //     } catch(\Exception $e){
    //         $response->setError(new Response\Error($e->getMessage(), $e->getCode()));        
    //     }
    // }
    /**
     * Salva a edição de um imposto relacionado a um serviço
    **/
    public function salvarEdicaoImposto(Request\Row $request, Response $response){
        try{
            $row   = $request->getRow();
            $nrorg = $this->environment->getNrorg();
            $cdimposto     = $row["CDIMPOSTO"];
            $vrprecimposto = $row["VRPRECIMPOSTO"];
            $this->impostoservice->salvarEdicaoImposto($cdimposto, $nrorg, $vrprecimposto);
        } catch(\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));        
        }
    }
}