<?php
namespace SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class SmPropostasInvestimentosFactory
{

    use TQuery;

    protected $connection;

    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }
    public function getInvestimentoproposta()
    {
        $sql = "SELECT 
                     fi.NRINVESTIMENTO,
                     fi.DSINVESTIMENTO,
                     fi.CDCATINVEST,
                     fc.NMCATINVEST,
                     fi.VRINVESTIMENTO,
                     fi.QTMESARMOTIZACAO,
                     fi.VRPERCJUROS,
                     fi.PERIODOJUROS,
                     fi.TPAMORTIZACAO,
                     fi.DTINCLUSAO,
                     fi.NRORG,
                     fi.DTULTATU,
                     fi.NRORGINCLUSAO,
                     fi.CDOPERINCLUSAO,
                     fi.NRORGULTATU,
                     fi.CDOPERULTATU,
                     fi.IDATIVO
                 FROM FCT_INVESTIMENTO fi
                 LEFT JOIN FCT_CATINVESTIMENTO fc ON fi.CDCATINVEST = fc.CDCATINVEST";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }


    public function getFormacaoPreco()
    {
        $sql = "SELECT NRFORMPRECOSERVICO, NMFORMPRECOSERVICO
                 FROM FCT_FORMPRECOSERVICO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTabFormacaoPreco($params)
    {
        $sql = "SELECT 
                    NRRELACFORMINVEST,
                    NRINVESTIMENTO,
                    NRFORMPRECOSERVICO,
                    NRORG,
                    DTINCLUSAO,
                    DTULTATU,
                    NRORGINCLUSAO,
                    CDOPERINCLUSAO,
                    NRORGULTATU,
                    CDOPERULTATU,
                    IDATIVO
                FROM FCT_RELACFORMINVESTIMENTO
                WHERE NRINVESTIMENTO = :NRINVESTIMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }

    public function getComposicaoInvestimento($params)
    {
        $sql = "SELECT FCT_RELACSERVINVESTIMENTO.NRRELACSERVINVEST,
            FCT_RELACSERVINVESTIMENTO.NRINVESTIMENTO,
            FCT_RELACSERVINVESTIMENTO.NRCOMPSERVICO,
            FCT_COMPSERVICO.NMCOMPSERVICO,
            FCT_COMPSERVICO.NRCOMPSERVICOPAI,
            COMP_PAI.NMCOMPSERVICO AS NMCOMPSERVICOPAI,
            FCT_RELACSERVINVESTIMENTO.NRORG,
            FCT_RELACSERVINVESTIMENTO.DTINCLUSAO,
            FCT_RELACSERVINVESTIMENTO.NRORGINCLUSAO,
            FCT_RELACSERVINVESTIMENTO.CDOPERINCLUSAO,
            OPINC.NMOPERADOR AS NMOPERINCLUSAO,
            FCT_RELACSERVINVESTIMENTO.DTULTATU,
            FCT_RELACSERVINVESTIMENTO.NRORGULTATU,
            FCT_RELACSERVINVESTIMENTO.CDOPERULTATU,
            OPATU.NMOPERADOR AS NMOPERULTATU,
            FCT_RELACSERVINVESTIMENTO.IDATIVO
        FROM
            FCT_RELACSERVINVESTIMENTO
        INNER JOIN FCT_COMPSERVICO ON FCT_RELACSERVINVESTIMENTO.NRCOMPSERVICO = FCT_COMPSERVICO.NRCOMPSERVICO
        AND FCT_RELACSERVINVESTIMENTO.NRORG = FCT_COMPSERVICO.NRORG
        LEFT JOIN FCT_COMPSERVICO COMP_PAI ON FCT_COMPSERVICO.NRCOMPSERVICOPAI = COMP_PAI.NRCOMPSERVICO
        AND FCT_COMPSERVICO.NRORG = COMP_PAI.NRORG
        LEFT JOIN OPERADOR OPINC ON FCT_RELACSERVINVESTIMENTO.CDOPERINCLUSAO = OPINC.CDOPERADOR
        AND FCT_RELACSERVINVESTIMENTO.NRORG = OPINC.NRORG
        LEFT JOIN OPERADOR OPATU ON FCT_RELACSERVINVESTIMENTO.CDOPERULTATU = OPATU.CDOPERADOR
        AND FCT_RELACSERVINVESTIMENTO.NRORG = OPATU.NRORG
        WHERE FCT_RELACSERVINVESTIMENTO.NRINVESTIMENTO = :NRINVESTIMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $params);
    }
    public function inserirRelacServInvest(array $params)
    {
        $sql = "INSERT INTO FCT_RELACSERVINVESTIMENTO 
                (NRRELACSERVINVEST, 
                 NRINVESTIMENTO, 
                 NRCOMPSERVICO, 
                 NRCOMPSERVICOPAI, 
                 NRORG, 
                 DTINCLUSAO, 
                 DTULTATU, 
                 NRORGINCLUSAO, 
                 CDOPERINCLUSAO, 
                 NRORGULTATU, 
                 CDOPERULTATU, 
                 IDATIVO)
        VALUES (:NRRELACSERVINVEST, 
                :NRINVESTIMENTO, 
                :NRCOMPSERVICO, 
                :NRCOMPSERVICOPAI, 
                :NRORG, 
                SYSDATE,
                NULL,
                :NRORGINCLUSAO, 
                :CDOPERINCLUSAO,  
                :NRORGULTATU, 
                :CDOPERULTATU, 
                :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarRelacServInvest(array $params)
    {
        $sql = "UPDATE FCT_RELACSERVINVESTIMENTO SET 
                NRINVESTIMENTO = :NRINVESTIMENTO, 
                NRCOMPSERVICO = :NRCOMPSERVICO, 
                NRCOMPSERVICOPAI = :NRCOMPSERVICOPAI, 
                NRORG = :NRORG, 
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU, 
                CDOPERULTATU = :CDOPERULTATU, 
                IDATIVO = :IDATIVO
            WHERE NRRELACSERVINVEST = :NRRELACSERVINVEST";

        $this->connection->executeStatement($sql, $params);
    }


    public function inserirInvestimento(array $params)
    {
        $sql = "INSERT INTO FCT_INVESTIMENTO 
                    (NRINVESTIMENTO,
                     DSINVESTIMENTO,
                     CDCATINVEST,
                     VRINVESTIMENTO,
                     QTMESARMOTIZACAO,
                     DTINCLUSAO,
                     NRORG,
                     DTULTATU,
                     NRORGINCLUSAO,
                     CDOPERINCLUSAO,
                     NRORGULTATU,
                     CDOPERULTATU,
                     IDATIVO)
             VALUES (:NRINVESTIMENTO,
                     :DSINVESTIMENTO,
                     :CDCATINVEST,
                     :VRINVESTIMENTO,
                     :QTMESARMOTIZACAO,
                     SYSDATE,
                     :NRORG,
                     NULL,
                     :NRORGINCLUSAO,
                     :CDOPERINCLUSAO,
                     NULL,
                     NULL,
                     :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarInvestimento(array $params)
    {
        $sql = "UPDATE FCT_INVESTIMENTO SET 
                DSINVESTIMENTO = :DSINVESTIMENTO,
                CDCATINVEST = :CDCATINVEST,
                VRINVESTIMENTO = :VRINVESTIMENTO,
                QTMESARMOTIZACAO = :QTMESARMOTIZACAO,
                DTULTATU = SYSDATE,
                NRORGULTATU = :NRORGULTATU,
                CDOPERULTATU = :CDOPERULTATU,
                IDATIVO = :IDATIVO
            WHERE NRINVESTIMENTO = :NRINVESTIMENTO";

        $this->connection->executeStatement($sql, $params);
    }

    public function inserirRelacFormInvest(array $params)
    {
        $sql = "INSERT INTO FCT_RELACFORMINVESTIMENTO 
            (NRRELACFORMINVEST, 
             NRINVESTIMENTO, 
             NRFORMPRECOSERVICO, 
             NRORG, 
             DTINCLUSAO, 
             DTULTATU, 
             NRORGINCLUSAO, 
             CDOPERINCLUSAO, 
             NRORGULTATU, 
             CDOPERULTATU, 
             IDATIVO)
    VALUES (:NRRELACFORMINVEST, 
            :NRINVESTIMENTO, 
            :NRFORMPRECOSERVICO, 
            :NRORG, 
            SYSDATE,
            NULL,
            :NRORGINCLUSAO, 
            :CDOPERINCLUSAO,  
            :NRORGULTATU, 
            :CDOPERULTATU, 
            :IDATIVO)";

        $this->connection->executeStatement($sql, $params);
    }

    public function atualizarRelacFormInvest(array $params)
    {
        $sql = "UPDATE FCT_RELACFORMINVESTIMENTO SET 
            NRINVESTIMENTO = :NRINVESTIMENTO, 
            NRFORMPRECOSERVICO = :NRFORMPRECOSERVICO, 
            NRORG = :NRORG, 
            DTULTATU = SYSDATE,
            NRORGULTATU = :NRORGULTATU, 
            CDOPERULTATU = :CDOPERULTATU, 
            IDATIVO = :IDATIVO
        WHERE NRRELACFORMINVEST = :NRRELACFORMINVEST";

        $this->connection->executeStatement($sql, $params);
    }

    public function deleteInvest($parametros)
    {
        $sql2 = "DELETE FROM FCT_RELACFORMINVESTIMENTO
                WHERE NRRELACFORMINVEST IN (" . $parametros . ")";
        $this->connection->executeStatement($sql2);
        $sql1 = "DELETE FROM FCT_RELACSERVINVESTIMENTO
                WHERE NRRELACSERVINVEST IN (" . $parametros . ")";
        $this->connection->executeStatement($sql1);
        $sql = "DELETE FROM  FCT_INVESTIMENTO
                WHERE NRINVESTIMENTO IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

    public function deleteComposi($parametros)
    {
        $sql = "DELETE FROM FCT_RELACSERVINVESTIMENTO
                WHERE NRRELACSERVINVEST IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

    public function deleteFormPreco($parametros)
    {
        $sql = "DELETE FROM FCT_RELACFORMINVESTIMENTO
                WHERE NRRELACFORMINVEST IN (" . $parametros . ")";

        $this->connection->executeStatement($sql);
    }

}