<?php

namespace SM\Service;

use SM\Factory\SmPropostasAlimentacaoFactory;
use Teknisa\Libs\Util\Environment;
use SM\Service\FindService;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class SmPropostasAlimentacaoService
{
    protected $connection;
    protected $PropostaAlimentacaoFactory;
    protected $environment;

    protected $findService;

    protected $newCode;

    public function __construct(SmPropostasAlimentacaoFactory $PropostaAlimentacaoFactory, Environment $environment, FindService $findService, NewCode $newCode, EntityManager $entityManager)
    {
        $this->PropostaAlimentacaoFactory = $PropostaAlimentacaoFactory;
        $this->environment = $environment;
        $this->findService = $findService;
        $this->newCode = $newCode;
        $this->connection = $entityManager->getConnection();
    }

    public function getPropostasAlimentacao()
    {
        return $this->PropostaAlimentacaoFactory->getPropostasAlimentacao();
    }

    public function getEscopoFaturamentoProposta($params){
        return $this->PropostaAlimentacaoFactory->getEscopoFaturamentoProposta($params);
    }

    public function getTabServicosPropostas($nrtipoorcamento)
    {
        return $this->PropostaAlimentacaoFactory->getTabServicosPropostas($nrtipoorcamento);
    }

    public function getInvestimentosPropostaAlimentacao($params)
    {

        $params = array(
            "NRSEQORCAMENTO" => $params['NRSEQORCAMENTO']
        );
        return $this->PropostaAlimentacaoFactory->getInvestimentosPropostaAlimentacao($params);
    }

    public function getComposicoesServicos($nrtipoorcamento, $nrcomposicao){
        $nrtipoorcamento = implode(",", $nrtipoorcamento);
        $nrcomposicao    = implode(",", array_unique($nrcomposicao));
       return $this->PropostaAlimentacaoFactory->getElementosComposicaoServico($nrtipoorcamento, $nrcomposicao);
    }


    public function getComposicaoInvestimento($params)
    {

        $params = array(
            "NRINVESTIMENTO" => $params['NRINVESTIMENTO']
        );
        return $this->investimentoFactory->getComposicaoInvestimento($params);
    }

    public function getTabFormacaoPreco($params)
    {
        $params = array(
            "NRINVESTIMENTO" => $params['NRINVESTIMENTO']
        );
        return $this->investimentoFactory->getTabFormacaoPreco($params);
    }

    public function getFormacaoPreco()
    {
        return $this->investimentoFactory->getFormacaoPreco();
    }
   
    public function salvarPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            $nrSeqOrcamento = $this->newCode->getCodeByTableName("FCT_ORCAMENTO", $nrorg, 1, 10, true);
            if (empty($params["NRSEQORCAMENTO"]) && $params["NRSEQORCAMENTO"] == null) {
                $parametrosProposta = array(
                    "NRSEQORCAMENTO" => $nrSeqOrcamento,
                    "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                    "NRVERSAOORCAMENTO" => $params["NRVERSAOORCAMENTO"],
                    "DTREFORCAMENTO" => $params["DTREFORCAMENTO"],
                    "IDSITUACAOORCAMENTO" => $params["IDSITUACAOORCAMENTO"],
                    "VRORCAMENTO" => $params["VRORCAMENTO"],
                    "DTACEITEORCA" => $params["DTACEITEORCA"],
                    "NRPARCNEGOCIO" => $params["NRPARCNEGOCIO"],
                    "VRPERCMARKUP" => $params["VRPERCMARKUP"],
                    "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                    "DTVALIDADE" => $params["DTVALIDADE"],
                    "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "IDATIVO" => 'S',
                );      
                
                $this->PropostaAlimentacaoFactory->inserirProposta($parametrosProposta);
            } else {
                $parametrosProposta = array(
                   "NRSEQORCAMENTO" => isset($params["NRSEQORCAMENTO"]),
                    "NRORCAMENTOMODELO" => $params["NRORCAMENTOMODELO"],
                    "NRVERSAOORCAMENTO" => $params["NRVERSAOORCAMENTO"],
                    "DTREFORCAMENTO" => $params["DTREFORCAMENTO"],
                    "IDSITUACAOORCAMENTO" => $params["IDSITUACAOORCAMENTO"],
                    "VRORCAMENTO" => $params["VRORCAMENTO"],
                    "DTACEITEORCA" => $params["DTACEITEORCA"],
                    "NRPARCNEGOCIO" => $params["NRPARCNEGOCIO"],
                    "VRPERCMARKUP" => isset($params["VRPERCMARKUP"]),
                    "QTDIASVALIDADE" => $params["QTDIASVALIDADE"],
                    "DTVALIDADE" => $params["DTVALIDADE"],
                    "NRDIASCARENCIA" => $params["NRDIASCARENCIA"],
                    "NRORGULTATU"   => $nrorg,
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "CDOPERINCLUSAO" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarProposta($parametrosProposta);
            }
    
            // if (empty($params["NRFORMPRECOSERVICO"]) && $params["NRFORMPRECOSERVICO"] == null) {
            //     $nrCalcFormPrecServ = $this->newCode->getCodeByTableName("FCT_CALCFORMPRECSERV", $nrorg, 1, 10, true);
                
            //     $parametrosCalcFormPrecServ = array(
            //         "NRCALCFORMPRECSERV" => $nrCalcFormPrecServ,
            //         "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //          "NRORG" => $nrorg,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->inserirCalcFormPrecServ($parametrosCalcFormPrecServ);
            // } else {
            //     $parametrosCalcFormPrecServ = array(
            //         "NRCALCFORMPRECSERV" => $params["NRCALCFORMPRECSERV"],
            //         "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->atualizarCalcFormPrecServ($parametrosCalcFormPrecServ);
            // }
    
            // if (isset($params["NRRELACSERVORCA"]) && empty($params["NRRELACSERVORCA"]) && $params["NRRELACSERVORCA"] == null) {
            //     $parametrosServicoOrcamento = array(
            //         "NRRELACSERVORCA" => $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true),
            //         "QTSEGUNDA" => $params["QTSEGUNDA"],
            //         "QTTERCA" => $params["QTTERCA"],
            //         "QTQUARTA" => $params["QTQUARTA"],
            //         "QTQUINTA" => $params["QTQUINTA"],
            //         "QTSEXTA" => $params["QTSEXTA"],
            //         "QTSABADO" => $params["QTSABADO"],
            //         "QTDOMINGO" => $params["QTDOMINGO"],
            //         "NRORG" => $nrorg,
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->inserirRelacServOrcamento($parametrosServicoOrcamento);
            // } else {
            //     $parametrosServicoOrcamento = array(
            //         "NRRELACSERVORCA" => isset($params["NRRELACSERVORCA"]),
            //         "QTSEGUNDA" => $params["QTSEGUNDA"],
            //         "QTTERCA" => $params["QTTERCA"],
            //         "QTQUARTA" => $params["QTQUARTA"],
            //         "QTQUINTA" => $params["QTQUINTA"],
            //         "QTSEXTA" => $params["QTSEXTA"],
            //         "QTSABADO" => $params["QTSABADO"],
            //         "QTDOMINGO" => $params["QTDOMINGO"],
            //         "NRORG" => $nrorg,
            //         "NRORGINCLUSAO" => $nrorg,
            //         "CDOPERINCLUSAO" => $operador,
            //         "NRORGULTATU" => $nrorg,
            //         "CDOPERULTATU" => $operador,
            //         "IDATIVO" => 'S',
            //     );
            //     $this->PropostaAlimentacaoFactory->atualizarRelacServOrcamento($parametrosServicoOrcamento);
            // }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function salvarServicosPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
        
            if (empty($params["NRRELACSERVORCA"]) && $params["NRRELACSERVORCA"] == null) {
                $parametrosServicoOrcamento = array(
                    "NRRELACSERVORCA" => $this->newCode->getCodeByTableName("FCT_RELACSERVORCAMENTO", $nrorg, 1, 10, true),
                    "NRSEQORCAMENTO" => $params["NRSEQORCAMENTO"],
                    "QTSEGUNDA" => $params["QTSEGUNDA"],
                    "QTTERCA" => $params["QTTERCA"],
                    "QTQUARTA" => $params["QTQUARTA"],
                    "QTQUINTA" => $params["QTQUINTA"],
                    "QTSEXTA" => $params["QTSEXTA"],
                    "QTSABADO" => $params["QTSABADO"],
                    "QTDOMINGO" => $params["QTDOMINGO"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->inserirRelacServOrcamento($parametrosServicoOrcamento);
            } else {
                $parametrosServicoOrcamento = array(
                    "NRRELACSERVORCA" => isset($params["NRRELACSERVORCA"]),
                    "QTSEGUNDA" => $params["QTSEGUNDA"],
                    "QTTERCA" => $params["QTTERCA"],
                    "QTQUARTA" => $params["QTQUARTA"],
                    "QTQUINTA" => $params["QTQUINTA"],
                    "QTSEXTA" => $params["QTSEXTA"],
                    "QTSABADO" => $params["QTSABADO"],
                    "QTDOMINGO" => $params["QTDOMINGO"],
                    "NRORG" => $nrorg,
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarRelacServOrcamento($parametrosServicoOrcamento);
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveTabMaodeObra($params){
        $this->connection->beginTransaction();
        
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
           if (empty($params["NRCALCFORMPRECSERV"]) && $params["NRCALCFORMPRECSERV"] == null) {
                $nrCalcFormPrecServ = $this->newCode->getCodeByTableName("FCT_CALCFORMPRECSERV", $nrorg, 1, 10, true);
                
                $parametrosCalcFormPrecServ = array(
                    "NRCALCFORMPRECSERV" => $nrCalcFormPrecServ,
                    "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                     "NRORG" => $nrorg,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->inserirCalcFormPrecServ($parametrosCalcFormPrecServ);
            } else {
                $parametrosCalcFormPrecServ = array(
                    "NRCALCFORMPRECSERV" => $params["NRCALCFORMPRECSERV"],
                    "NRFORMPRECOSERVICO" => $params["NRFORMPRECOSERVICO"],
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador,
                    "NRORGULTATU" => $nrorg,
                    "CDOPERULTATU" => $operador,
                    "IDATIVO" => 'S',
                );
                $this->PropostaAlimentacaoFactory->atualizarCalcFormPrecServ($parametrosCalcFormPrecServ);
            }
              
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    
    }



// public function saveInvestimentoPropostaAlimentacao($params)
// {
//     $this->connection->beginTransaction();

//     try {
//         $nrorg = $this->environment->getNrorg();
//         $operador = $this->environment->getCdOperador();

//         foreach ($params["DRG"] as $row) {
//             if ($row["NRINVESTIMENTO"] != 0) {
                
//                 // Atualiza a relação investimento-proposta
//                 $paramInvestProposta = [
//                     "NRINVESTPROP" => $row["NRINVESTPROP"],
//                     "VRUNITINVESTORCA" => $row["VRINVESTIMENTO"],
//                     "VRTOTINVETORCA" => $row["VRINVESTIMENTO"] * $row["QTINVESTORCA"],
//                     "QTINVESTORCA" => $row["QTINVESTORCA"]
//                 ];
//                 $this->PropostaAlimentacaoFactory->atualizarInvestimento($paramInvestProposta);
//             } else {
//                 // Cria novo investimento
//                 // Insere a relação investimento-proposta
//                 $nrInvestProp = $this->newCode->getCodeByTableName("FCT_RELACINVESTPROPOSTA", $nrorg, 1, 10, true);
//                 $paramInvestProposta = [
//                         "NRINVESTPROP" =>  $nrInvestProp,
//                         "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
//                         "NRSEQORCAMENTO" => $row["NRSEQORCAMENTO"],
//                         "DSINVESTIMENTO" =>$row["DSINVESTIMENTO"],
//                         "VRUNITINVESTORCA" => $row["VRUNITINVESTORCA"],
//                         "QTMESARMOTIZACAO" => $row["QTMESARMOTIZACAO"],
//                         "NRORG" => $nrorg,
//                         "NRORGINCLUSAO" => $nrorg,
//                         "CDOPERINCLUSAO" => $operador,
//                         "IDATIVO" => $row["IDATIVO"]
//                 ];
//                 $this->PropostaAlimentacaoFactory->inserirInvestimento($paramInvestProposta);
//             }
//         }

//         $this->connection->commit();
//     } catch (\Exception $e) {
//         $this->connection->rollback();
//         throw $e;
//     }
// }

    public function saveDespesasPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
    
            foreach ($params["DRG"] as $row) {
                if ($row["CDDESPESA"] != 0) {
                    $parametros = array(
                        "CDDESPESA"       => $row["CDDESPESA"],
                        "DSDESPESA"       => $row["DSDESPESA"],
                        "VRDESPESA"       => $row["VRDESPESA"],
                        "CDPERIODO"       => $row["CDPERIODO"],
                        "IDATIVO"         => $row["IDATIVO"],
                        "NRORGULTATU"     => $nrorg,
                        "CDOPERULTATU"    => $operador
                    );
    
                    $this->PropostaAlimentacaoFactory->atualizarDespesa($parametros);
                } else {
                    $cdDespesa = $this->newCode->getCodeByTableName("FCT_DESPESA", $nrorg, 1, 10, true);
    
                    $parametros = array(
                        "CDDESPESA"       => $cdDespesa,
                        "DSDESPESA"       => $row["DSDESPESA"],
                        "VRDESPESA"       => $row["VRDESPESA"],
                        "CDPERIODO"       => $row["CDPERIODO"],
                        "NRORG"           => $nrorg,
                        "NRORGINCLUSAO"   => $nrorg,
                        "CDOPERINCLUSAO"  => $operador,
                        "IDATIVO"         => $row["IDATIVO"]
                    );
    
                    $this->PropostaAlimentacaoFactory->inserirDespesa($parametros);
                }
            }
    
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }  

    public function saveImpostoPropostaAlimentacao($params)
    {
        $this->connection->beginTransaction();
    
        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            
            foreach ($params["DRG"] as $row) {
                if ($row["NRIMPOSTOSERV"] != 0) {
                    $parametros = array(
                        "CDIMPOSTO" => $row["CDIMPOSTO"],
                        "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                        "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => "S",
                        "NRIMPOSTOSERV" => $row["NRIMPOSTOSERV"],
                    );
                    
                    $this->PropostaAlimentacaoFactory->updateTabImposto($parametros);
                } else {
                    $NRIMPOSTOSERV = $this->newCode->getCodeByTableName("FCT_RELACIMPOSTOSERV", $nrorg, 1, 10, true);
                    
                    $parametros = array(
                    "NRIMPOSTOSERV" =>  $NRIMPOSTOSERV,
                    "CDIMPOSTO" =>     $row["CDIMPOSTO"],
                    "VRPRECIMPOSTO" => $row["VRPRECIMPOSTO"],
                    "NRCOMPSERVICO" => isset($row["NRCOMPSERVICO"]),
                    "NRORG" => $nrorg,
                    "IDATIVO" => "S",
                    "NRORGINCLUSAO" => $nrorg,
                    "CDOPERINCLUSAO" => $operador
                    );
                    
                    $this->PropostaAlimentacaoFactory->saveTabImposto($parametros);
                }
            }
            
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveComposicaoServicos($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();
            foreach ($params["DRG"] as $row) {
                if ($row["NRRELACSERVINVEST"] != 0) {
                    $parametros = array(
                        "NRRELACSERVINVEST" => $row["NRRELACSERVINVEST"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => $row["NRCOMPSERVICOPAI"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->atualizarRelacServInvest($parametros);
                } else {
                    $nrRelacServInvest = $this->newCode->getCodeByTableName("FCT_RELACSERVINVESTIMENTO", $nrorg, 1, 10, true);
                    $parametros = array(
                        "NRRELACSERVINVEST" => $nrRelacServInvest,
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRCOMPSERVICO" => $row["NRCOMPSERVICO"],
                        "NRCOMPSERVICOPAI" => isset($row["NRCOMPSERVICOPAI"]),
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "IDATIVO" => $row["IDATIVO"],
                        "CDOPERULTATU" => $operador
                    );

                    $this->investimentoFactory->inserirRelacServInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function saveRelacFormInvest($params)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operador = $this->environment->getCdOperador();

            foreach ($params["DRG"] as $row) {
                if ($row["NRRELACFORMINVEST"] != 0) {
                    $parametros = array(
                        "NRRELACFORMINVEST" => $row["NRRELACFORMINVEST"],
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->atualizarRelacFormInvest($parametros);
                } else {
                    $nrRelacFormInvest = $this->newCode->getCodeByTableName("FCT_RELACFORMINVESTIMENTO", $nrorg, 1, 10, true);

                    $parametros = array(
                        "NRRELACFORMINVEST" => $nrRelacFormInvest,
                        "NRINVESTIMENTO" => $row["NRINVESTIMENTO"],
                        "NRFORMPRECOSERVICO" => $row["NRFORMPRECOSERVICO"],
                        "NRORG" => $nrorg,
                        "NRORGINCLUSAO" => $nrorg,
                        "CDOPERINCLUSAO" => $operador,
                        "NRORGULTATU" => $nrorg,
                        "CDOPERULTATU" => $operador,
                        "IDATIVO" => $row["IDATIVO"]
                    );

                    $this->investimentoFactory->inserirRelacFormInvest($parametros);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteInvest($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);

            $this->investimentoFactory->deleteFormPreco($retorno);
            $this->investimentoFactory->deleteComposi($retorno);

            $this->investimentoFactory->deleteInvest($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }


    public function deleteComposi($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->investimentoFactory->deleteComposi($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteFormPreco($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $retorno = $this->findService->formatarArray($parametros["NRS"]);
            $this->investimentoFactory->deleteFormPreco($retorno);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}