import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    Button,
    DialogService, Form, LoadingService, Modal, ModalService, Select, SelectMultiple, Tabs, Textarea, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import {
    ZdNumber,
    ZdSelect, ZdTextarea, ZdTextInput,
} from '@zeedhi/vuetify';

export class SmPropostasAlimentacaoController {
    public tekgridVisivel: boolean = true;

    public currentRow: any = {};

    public viewTab: boolean = false;

    public tituloHeaderFrameOperadoresEquipes: String = '';

    private ModalFiltro!: Modal;

    get textFrame() {
        return this.currentRow.NRORCAMENTO;
    }

    get formVisivel() {
        return !this.tekgridVisivel;
    }

    get tabInvisivel() {
        return this.currentRow.IDTPMEDIDASERV !== 'F';
    }

    public addNewRowImposto({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRIMPOSTOSERV: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`CDIMPOSTO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowDespesa({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ CDDESPESA: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSDESPESA_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowInvestimento({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRINVESTIMENTO: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSINVESTIMENTO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public AddNewRowMaoObra({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCALCFORMPRECSERV: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRCALCFORMPRECSERV_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public async createModalFiltroElemento() {
        this.ModalFiltro = await ModalService.createFromJson('/investimentos/SmpropostasFiltro', true);
        // this.openModalFiltroElemento();
    }

    public openModalFiltroElemento() {
        this.ModalFiltro.show();
        setTimeout(() => {
            Metadata.getInstance<Select>('filtro-DSINVESTIMENTO').value = this.currentRow.DSINVESTIMENTO;
            Metadata.getInstance<Select>('filtro-categoria').value = this.currentRow.CDCATINVEST;
        }, 200);
    }

    public recarregarTabs() {
        const voltarButton = Metadata.getInstance<Button>('buttonContinue-4');
        const avancarButton = Metadata.getInstance<Button>('buttonContinue-3');
        const salvarButton = Metadata.getInstance<Button>('buttonContinue-2');

        // Esconder o botão "Voltar" apenas no primeiro step
        voltarButton.isVisible = this.activeStep !== 1;

        // Exibir "Salvar" e esconder "Avançar" apenas no step 9
        if (this.activeStep === 9) {
            salvarButton.isVisible = true;
            avancarButton.isVisible = false;
        } else {
            salvarButton.isVisible = false;
            avancarButton.isVisible = true; // <-- Aqui garantimos que "Avançar" reapareça nos outros steps
        }
    }

    public changeEscopoFaturamento({ component }: IEventParam<Select>) {
        const escopoFaturamento = Metadata.getInstance<Select>('select-escopo-fatura');

        if (component.value) {
            escopoFaturamento.disabled = false;
            escopoFaturamento.datasource.setFilter({
                NRORCAMENTOMODELO: component.value,
            });
        } else {
            escopoFaturamento.value = null;
            escopoFaturamento.disabled = true;
        }
    }

    // public async carregarEscopoFaturamento(modeloSelecionado: string, ) {
    //     const escopoSelect = Metadata.getInstance<Select>('select-escopo-fatura');

    //     if (!modeloSelecionado) {
    //         escopoSelect.response.data = [];
    //         return;
    //     }

    //     try {
    //         const response = await Http.get(`/findEscopoFaturamento?nrOrcamentoModelo=${modeloSelecionado}`);

    //         if (response && response.data) {
    //             escopoSelect.data = response.data;
    //         }
    //     } catch (error) {
    //         console.error("Erro ao buscar Escopo de Faturamento:", error);
    //         escopoSelect.data = [];
    //     }
    // }

    public closeModalFiltroElemento() {
        this.ModalFiltro.hide();
    }

    public eventFilter({ event }: any) {
        this.openModalFiltroElemento();
        event.preventDefault();
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRINVESTIMENTO: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`DSINVESTIMENTO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowComposi({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRRELACSERVINVEST: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRCOMPSERVICO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowFormPreco({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRRELACFORMINVEST: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NRFORMPRECOSERVICO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    private clickTimeout: any = null;

    private clicks: number = 0;

    public rowClick({ row, component }: any) {
        this.clicks += 1;

        if (this.clicks === 1) {
            this.clickTimeout = setTimeout(() => {
                if (!component.editing) { // Verifica se não está editando antes de abrir o frame
                    this.currentRow = row;
                    this.tekgridVisivel = false;

                    const tabs = Metadata.getInstance<Tabs>('tabsInformation');

                    // Verifica se a tab já está ativa antes de reabrir
                    if (tabs.activeTab !== 0) {
                        tabs.activeTab = 0;
                    }

                    // Garante que os filtros corretos são aplicados
                    setTimeout(() => {
                        this.setFilterOperadorEquipe();
                        this.setFilterFormpreco();
                    }, 100);
                }

                this.clicks = 0;
            }, 300);
        } else if (this.clicks === 2) {
            clearTimeout(this.clickTimeout);
            component.editing(row);
            this.clicks = 0;
        }
    }

    public setFilterFormpreco() {
        const gridComposicao = Metadata.getInstance<TekGrid>('basic-tek-grid-investimento');
        gridComposicao.datasource.setFilter({
            NRSEQORCAMENTO: this.currentRow.NRSEQORCAMENTO,
        });
    }

    public abrirFrameAdd({ row }: any) {
        this.tekgridVisivel = false;
        this.currentRow = row;
    }

    private fecharFrame() {
        this.tekgridVisivel = true; // Exibir novamente o grid principal
        this.currentRow = null; // Resetar a linha atual para evitar inconsistências
    }

    public async abrirFrame({ row }: any) {
        console.warn('Row recebido:', row);
        this.tekgridVisivel = false;
        this.currentRow = row;
        this.setFilterFormpreco();
        await this.carregarDadosEscopoFaturamentoTabEscopoFaturamento();
    }

    public carregarDadosEscopoFaturamentoTabEscopoFaturamento() {
        console.warn(this.currentRow);

        Metadata.getInstance<Select>('NRORCAMENTOMODELO').value = this.currentRow.NRORCAMENTOMODELO;
        Metadata.getInstance<Select>('select-escopo-fatura').value = this.currentRow.NRTIPOORCAMENTO;
        Metadata.getInstance<TextInput>('NRCALCFORMPRECSERV').value = this.currentRow.NRCALCFORMPRECSERV;
        Metadata.getInstance<TextInput>('NRVERSAOORCAMENTO').value = this.currentRow.NRVERSAOORCAMENTO;
        Metadata.getInstance<TextInput>('DTREFORCAMENTO').value = this.currentRow.DTREFORCAMENTO || new Date().toISOString().split('T')[0];
        Metadata.getInstance<Select>('IDSITUACAOORCAMENTO').value = this.currentRow.IDSITUACAOORCAMENTO;
        Metadata.getInstance<TextInput>('VRORCAMENTO').value = this.currentRow.VRORCAMENTO;
        Metadata.getInstance<TextInput>('DTACEITEORCA').value = this.currentRow.DTACEITEORCA;
        Metadata.getInstance<Select>('CDOPERRESPONSAVEL').value = this.currentRow.CDOPERRESPONSAVEL;
        Metadata.getInstance<SelectMultiple>('NRPARCNEGOCIO').value = this.currentRow.NRPARCNEGOCIO;
        Metadata.getInstance<TextInput>('NRINSCRICAOPARC').value = this.currentRow.NRINSCRICAOPARC;
        Metadata.getInstance<TextInput>('VRPERCMARKUP').value = this.currentRow.VRPERCMARKUP;
        Metadata.getInstance<SelectMultiple>('DSREFERENCIAORCA').value = this.currentRow.DSREFERENCIAORCA;
        Metadata.getInstance<TextInput>('QTDIASVALIDADE').value = this.currentRow.QTDIASVALIDADE;
        Metadata.getInstance<TextInput>('DTVALIDADE').value = this.currentRow.DTVALIDADE;
        Metadata.getInstance<TextInput>('NRDIASCARENCIA').value = this.currentRow.NRDIASCARENCIA;
        Metadata.getInstance<Textarea>('DSOBSERVACAOORCA').value = this.currentRow.DSOBSERVACAOORCA;
        Metadata.getInstance<Textarea>('DSJUSTIFCANCRENEG').value = this.currentRow.DSJUSTIFCANCRENEG;
        // Metadata.getInstance<TextInput>('HRENTRADA').value = this.currentRow.HRENTRADA;
        // Metadata.getInstance<TextInput>('HRSAIDA').value = this.currentRow.HRSAIDA;
        Metadata.getInstance<TextInput>('QTSEGUNDA').value = this.currentRow.QTSEGUNDA;
        Metadata.getInstance<TextInput>('QTTERCA').value = this.currentRow.QTTERCA;
        Metadata.getInstance<TextInput>('QTQUARTA').value = this.currentRow.QTQUARTA;
        Metadata.getInstance<TextInput>('QTQUINTA').value = this.currentRow.QTQUINTA;
        Metadata.getInstance<TextInput>('QTSEXTA').value = this.currentRow.QTSEXTA;
        Metadata.getInstance<TextInput>('QTSABADO').value = this.currentRow.QTSABADO;
        Metadata.getInstance<TextInput>('QTDOMINGO').value = this.currentRow.QTDOMINGO;
        Metadata.getInstance<TextInput>('NMFORMPRECOSERVICO').value = this.currentRow.NMFORMPRECOSERVICO;
        Metadata.getInstance<TextInput>('QTCALCULO').value = this.currentRow.QTCALCULO;
        Metadata.getInstance<TextInput>('VRUNITARIO').value = this.currentRow.VRUNITARIO;
        Metadata.getInstance<TextInput>('VRTOTAL').value = this.currentRow.VRTOTAL;
        Metadata.getInstance<Select>('NRFORMPRECOSERVICO').value = this.currentRow.NRFORMPRECOSERVICO;
    }

    public async salvarProposta() {
        const form = Metadata.getInstance<Form>('escopoFaturamentoTab-formRowDadosEscopoFaturamento');

        if (Utilities.validateForm(form)) {
            LoadingService.show();

            Http.post('/salvarPropostaAlimentacao', {
                NRSEQORCAMENTO: Metadata.getInstance<ZdTextInput>('NRSEQORCAMENTO').value,
                NRORCAMENTOMODELO: Metadata.getInstance<ZdSelect>('NRORCAMENTOMODELO').value,
                NRCALCFORMPRECSERV: Metadata.getInstance<ZdNumber>('NRCALCFORMPRECSERV').value,
                NRVERSAOORCAMENTO: Metadata.getInstance<ZdNumber>('NRVERSAOORCAMENTO').value,
                DTREFORCAMENTO: Metadata.getInstance<ZdTextInput>('DTREFORCAMENTO').value,
                IDSITUACAOORCAMENTO: Metadata.getInstance<ZdSelect>('IDSITUACAOORCAMENTO').value,
                VRORCAMENTO: Metadata.getInstance<ZdTextInput>('VRORCAMENTO').value,
                DTACEITEORCA: Metadata.getInstance<ZdTextInput>('DTACEITEORCA').value,
                // CDOPERRESPONSAVEL: Metadata.getInstance<ZdSelect>('CDOPERRESPONSAVEL').value,
                NRPARCNEGOCIO: Metadata.getInstance<Select>('NRPARCNEGOCIO').value,
                NRINSCRICAOPARC: Metadata.getInstance<ZdTextInput>('NRINSCRICAOPARC').value,
                VRPERCMARKUP: Metadata.getInstance<ZdTextInput>('VRPERCMARKUP').value,
                QTDIASVALIDADE: Metadata.getInstance<ZdTextInput>('QTDIASVALIDADE').value,
                DTVALIDADE: Metadata.getInstance<ZdTextInput>('DTVALIDADE').value,
                NRDIASCARENCIA: Metadata.getInstance<ZdTextInput>('NRDIASCARENCIA').value,
                DSOBSERVACAOORCA: Metadata.getInstance<ZdTextarea>('DSOBSERVACAOORCA').value,
                DSJUSTIFCANCRENEG: Metadata.getInstance<ZdTextarea>('DSJUSTIFCANCRENEG').value,
            })
                .then(() => {
                    Utilities.showSuccessMessage();
                    this.fecharFrame();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    LoadingService.hide();
                    this.tekgridVisivel = true;
                    this.fecharFrame();
                });
        }
    }

    public async salvarServicos() {
        const form = Metadata.getInstance<Form>('form-dados-servico-porposta=alimentacao');

        if (Utilities.validateForm(form)) {
            LoadingService.show();

            Http.post('/salvarServicosPropostaAlimentacao', {
                NRRELACSERVORCA: Metadata.getInstance<ZdTextInput>('NRRELACSERVORCA').value,
                QTSEGUNDA: Metadata.getInstance<ZdTextInput>('QTSEGUNDA').value,
                QTTERCA: Metadata.getInstance<ZdTextInput>('QTTERCA').value,
                QTQUARTA: Metadata.getInstance<ZdTextInput>('QTQUARTA').value,
                QTQUINTA: Metadata.getInstance<ZdTextInput>('QTQUINTA').value,
                QTSEXTA: Metadata.getInstance<ZdTextInput>('QTSEXTA').value,
                QTSABADO: Metadata.getInstance<ZdTextInput>('QTSABADO').value,
                QTDOMINGO: Metadata.getInstance<ZdTextInput>('QTDOMINGO').value,
            })
                .then(() => {
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    LoadingService.hide();
                });
        }
    }

    public saveTabImposto({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveImpostoPropostaAlimentacao', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public saveTabMaodeObra({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveTabMaodeObra', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    /* eslint-disable */


    // public saveFrameInvestimentos({ component, event }: IEventParam<TekGrid>, nr: any) {
    //     event?.preventDefault();
    //     if (Utilities.validateGrid(component)) {
    //         LoadingService.show();
    //         const editedRows = component.getEditedRows().map(row => ({
    //             ...row,
    //             NRINVESTPROP: this.currentRow.NRINVESTPROP,
    //             NRSEQORCAMENTO: this.currentRow. NRSEQORCAMENTO,
    //         }));
    //         const sendObject = {
    //             DRG: editedRows
    //         };
    //         console.warn(editedRows);
    //         Http.post('/saveInvestimentoPropostaAlimentacao', sendObject)
    //             .then(() => {
    //                 component.datasource.get();
    //                 Utilities.showSuccessMessage();
    //             })
    //             .catch((error: any) => {
    //                 Utilities.showErrorMessage(error);
    //             })
    //             .finally(() => {
    //                 component.cancelEditedRows();
    //                 LoadingService.hide();
    //             });
    //     }
    // }

    /* eslint-enable */

    public saveFrameInvestimentos({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveInvestimentoPropostaAlimentacao', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public saveFrameDespesas({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveDespesasPropostaAlimentacao', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public filtrarElemento() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-investimento');
        const form = Metadata.getInstance<Form>('form-filter');

        const selectedInvestimentos = Metadata.getInstance<SelectMultiple>('filtro-DSINVESTIMENTO').value;
        const selectedCategorias = Metadata.getInstance<SelectMultiple>('filtro-categoria').value;

        // Buscar os nomes dos investimentos selecionados
        const investimentosSelecionados = Metadata.getInstance<SelectMultiple>('filtro-DSINVESTIMENTO')
            .datasource.data
            .filter((item: any) => selectedInvestimentos.includes(item.NRINVESTIMENTO))
            .map((item: any) => item.DSINVESTIMENTO);

        this.currentRow.CDCATINVEST = selectedCategorias;
        this.currentRow.DSINVESTIMENTO = investimentosSelecionados;

        if (Utilities.validateForm(form)) {
            grid.datasource.setFilter({
                CDCATINVEST: this.currentRow.CDCATINVEST,
                DSINVESTIMENTO: this.currentRow.DSINVESTIMENTO,
            });
            this.ModalFiltro.hide();
        }
    }

    public activeStep: number = 1;

    public steps: number = 9;

    public step: number = 1;

    /* eslint-disable */

    public nextStep() {
        if (this.activeStep < this.steps) {
            this.activeStep++;
        }
        this.recarregarTabs();
    }

    public backStep() {
        if (this.activeStep > 1) {
            this.activeStep--;
        }
        this.recarregarTabs(); // Atualiza a visibilidade dos botões
    }

    /* eslint-enable */

    public limparCampos() {
        Metadata.getInstance<Form>('form-filter').clearForm();
    }

    public verificaTab() {
        const tabs = Metadata.getInstance<Tabs>('tabsInformation');
        const tabAtiva = tabs.activeTab;

        if (tabs.tabs[tabAtiva].name === 'tab_variacao') {
            this.carregarDadosAbaVariacao();
        }
    }

    public voltarTekGrid() {
        this.tekgridVisivel = true;
    }

    public setFilterOperadorEquipe() {
        const gridOperador = Metadata.getInstance<TekGrid>('tab-composi');
        gridOperador.datasource.setFilter({
            NRINVESTIMENTO: this.currentRow.NRINVESTIMENTO,
        });
    }

    public investimentoTabComposicoes() {
        console.warn('entrou');
        const grid = Metadata.getInstance<TekGrid>('tab-composi');
        grid.datasource.setFilter({
            NRINVESTIMENTO: this.currentRow.NRINVESTIMENTO,
        });
        console.warn('ede');
    }

    public carregarDadosAbaVariacao() {
        const cdtpmedidaserv = this.currentRow.NRIMPOSTOSERV;
        try {
            const grid = Metadata.getInstance<TekGrid>('tabcomposi');
            grid.datasource.setFilter({
                CDIMPOSTO: cdtpmedidaserv,
            });
        } catch (e) {
            // console.log(e);
        }
    }

    public carregarDadosAbaTipoMedida() {
        Metadata.getInstance<Select>('CDIMPOSTO').value = this.currentRow.CDIMPOSTO;
        Metadata.getInstance<TextInput>('VRPRECIMPOSTO').value = this.currentRow.VRPRECIMPOSTO;
        Metadata.getInstance<Select>('NRCOMPSERVICO').value = this.currentRow.NRCOMPSERVICO;
    }

    public saveInvestimentos({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveInvestimentos', sendObject).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
                LoadingService.hide();
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }
    /* eslint-disable */

    public saveComposiServ({ component, event }: IEventParam<TekGrid>, nr: any) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const editedRows = component.getEditedRows().map(row => ({
                ...row,
                NRINVESTIMENTO: this.currentRow.NRINVESTIMENTO,
            }));
            const sendObject = {
                DRG: editedRows
            };
            Http.post('/saveComposicaoInvest', sendObject)
                .then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
        }
    }

    public saveFormPreco({ component, event }: IEventParam<TekGrid>, nr: any) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const editedRows = component.getEditedRows().map(row => ({
                ...row,
                NRINVESTPROP: this.currentRow.NRINVESTPROP,
            }));
            const sendObject = {
                DRG: editedRows
            };
            Http.post('/saveRelacFormInvest', sendObject)
                .then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
        }
    }

    /* eslint-enable */

    public showDialogWithButtonsGrid({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasInvestimentosController.deleteInvest');
    }

    public deleteInvest() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-investimento');
        LoadingService.show();
        Http.post('/deleteInvest', {
            NRS: grid.selectedRows.map((element: any) => element.NRINVESTIMENTO),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public showDialogBeforeDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasInvestimentosController.deleteComposi');
    }

    public deleteComposi() {
        const grid = Metadata.getInstance<TekGrid>('tab-composi');
        LoadingService.show();
        Http.post('/deleteComposi', {
            NRS: grid.selectedRows.map((element: any) => element.NRRELACSERVINVEST),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public showDialogBeforeDeleteForm({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasInvestimentosController.deleteFormImposto');
    }

    public deleteFormImposto() {
        const grid = Metadata.getInstance<TekGrid>('tab-formPreco');
        LoadingService.show();
        Http.post('/deleteFormPreco', {
            NRS: grid.selectedRows.map((element: any) => element.NRRELACFORMINVEST),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
