import {
    Http, IEventParam, Metadata,
} from '@zeedhi/core';
import {
    DialogService, LoadingService, Select, Tabs, TextInput,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';

export class SmPropostasImpostosController {
    public tekgridVisivel: boolean = true;

    public currentRow: any = {};

    public viewTab: boolean = false;

    get textFrame() {
        return this.currentRow.NMIMPOSTO;
    }

    get formVisivel() {
        return !this.tekgridVisivel;
    }

    get tabInvisivel() {
        return this.currentRow.IDTPMEDIDASERV !== 'F';
    }

    public addNewRow({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRCOMPSERVICO: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`NMCOMPSERVICO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    public addNewRowImposto({ component }: IEventParam<TekGrid>) {
        const newId = 0;
        component.addNewRow({ NRIMPOSTOSERV: newId, IDATIVO: 'S' }, 'start');
        setTimeout(() => {
            const nameInput = Metadata.getInstance<TextInput>(`CDIMPOSTO_editable_${newId}`);
            nameInput.setFocus();
        }, 100);
    }

    private clickTimeout: any = null;

    private clicks: number = 0;

    public rowClick({ row, component }: any) {
        this.clicks += 1;

        if (this.clicks === 1) {
            this.clickTimeout = setTimeout(() => {
                if (!component.editing) {
                    this.currentRow = row;
                    this.tekgridVisivel = false;
                    const tabs = Metadata.getInstance<Tabs>('tabs-impostos');
                    tabs.activeTab = 0;
                }

                this.clicks = 0;
            }, 300);
        } else if (this.clicks === 2) {
            clearTimeout(this.clickTimeout);
            component.editing(row);

            this.clicks = 0;
        }
    }

    public setComposicaoServ() {
        const gridComposicao = Metadata.getInstance<TekGrid>('tab-variacao');
        gridComposicao.datasource.setFilter({
            NRINVESTIMENTO: this.currentRow.NRINVESTIMENTO,
        });
    }

    public afterModify() {
        Metadata.getInstance<TekGrid>('tabcomposi').datasource.get();
    }

    public abrirFrameAdd() {
        this.tekgridVisivel = false;
        const tabs = Metadata.getInstance<Tabs>('tabs-impostos');
        tabs.activeTab = 0;
        Metadata.getInstance<Select>('NRIMPOSTOSERV').value = null;
        Metadata.getInstance<TextInput>('VRPRECIMPOSTO').value = null;
    }

    public abrirFrame({ row }: any) {
        this.tekgridVisivel = false;
        this.currentRow = row;
        const tabs = Metadata.getInstance<Tabs>('tabs-impostos');
        tabs.activeTab = 0;
        this.carregarDadosAbaTipoMedida();
    }

    public verificaTab() {
        const tabs = Metadata.getInstance<Tabs>('tabs-impostos');
        const tabAtiva = tabs.activeTab;

        if (tabs.tabs[tabAtiva].name === 'tek-grid-composicao-imposto') {
            this.carregarDadosAbaVariacao();
        }
    }

    public voltarTekGrid() {
        this.tekgridVisivel = true;
    }

    public carregarDadosAbaVariacao() {
        const cdtpmedidaserv = this.currentRow.NRIMPOSTOSERV;
        try {
            const grid = Metadata.getInstance<TekGrid>('tabcomposi');
            grid.datasource.setFilter({
                CDIMPOSTO: cdtpmedidaserv,
            });
        } catch (e) {
            // console.log(e);
        }
    }

    public carregarDadosAbaTipoMedida() {
        Metadata.getInstance<Select>('CDIMPOSTO').value = this.currentRow.CDIMPOSTO;
        Metadata.getInstance<TextInput>('VRPRECIMPOSTO').value = this.currentRow.VRPRECIMPOSTO;
        Metadata.getInstance<Select>('NRCOMPSERVICO').value = this.currentRow.NRCOMPSERVICO;
    }

    /* eslint-disable */

    public saveTabImposto({ component, event }: IEventParam<TekGrid>) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const sendObject = {
                DRG: component.getEditedRows(),
            };
            Http.post('/saveTabImposto', sendObject, {
            }).then(() => {
                component.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                component.cancelEditedRows();
                LoadingService.hide();
            });
        }
    }

    public saveComposicaoservicotab({ component, event }: IEventParam<TekGrid>, nr: any) {
        event?.preventDefault();
        if (Utilities.validateGrid(component)) {
            LoadingService.show();
            const editedRows = component.getEditedRows().map(row => ({
                ...row,
                 CDIMPOSTO: this.currentRow. CDIMPOSTO,
                 NRIMPOSTOSERV: this. currentRow. NRIMPOSTOSERV,
            }));
            const sendObject = {
                DRG: editedRows
            };
            Http.post('/saveComposicaoServicoTab', sendObject)
                .then(() => {
                    component.datasource.get();
                    Utilities.showSuccessMessage();
                })
                .catch((error: any) => {
                    Utilities.showErrorMessage(error);
                })
                .finally(() => {
                    component.cancelEditedRows();
                    LoadingService.hide();
                });
        }
    }

    public showDialogWithButtonsGrid({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasImpostosController.deleteImposto');
    }

    public deleteImposto() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-imposto');
        LoadingService.show();
        Http.post('/deleteImposto', {
            NRS: grid.selectedRows.map((element: any) => element.NRIMPOSTOSERV),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }

    public showDialogWBeforeDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'SmPropostasImpostosController.deleteComposiImposto');
    }

    public deleteComposiImposto() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-composicao-imposto');
        LoadingService.show();
        Http.post('/deleteImpostoComposicaoServ', {
            NRS: grid.selectedRows.map((element: any) => element.NRCOMPSERVICO),
        }).then(() => {
            grid.selectedRows = [];
            grid.reload();
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
